/*
 * xcut.c    Manipulate X cut buffers from command line.
 *
 * Copyright (C) Tim Potter, 1998-1999
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: xcut.c,v 1.4 1999/09/21 05:55:33 tpot Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

#define BUFSIZE 65535                /* Buffer size for reading input */

/* Print program usage information */

void usage(char *progname)
{
    fprintf(stderr, "usage: %s [options]\n", progname);
    fprintf(stderr, "\t-display displayname\tserver to connect to\n");
    fprintf(stderr, "\t-p\t\t\tpaste cut buffer to stdout\n");
    exit(1);
}

/* Cut from stdin */

void cut(char *progname, Display *display)
{    
    char *file_buffer, *cut_buffer;
    int nread, cut_size;

    file_buffer = malloc(BUFSIZE);
    cut_buffer = NULL;

    cut_size = 0;

    /* Store all of standard input */

    while((nread = read(STDIN_FILENO, file_buffer, BUFSIZE)) > 0) {

	/* Add to cut buffer */

	cut_size += nread;
	cut_buffer = realloc(cut_buffer, cut_size);
	memcpy(&cut_buffer[cut_size - nread], file_buffer, nread);
    }

    /* Error processing */

    if (nread == -1) {
	perror(progname);
	exit(2);
    }

    /* Perform the cut.  If we don't call XSetSelectionOwner() before
       doing the cut, existing selections are not overridden. */

    XSetSelectionOwner(display, XA_PRIMARY, None, CurrentTime);
    XStoreBytes(display, cut_buffer, cut_size);

    /* Clean up */

    free(cut_buffer);
    free(file_buffer);
}

/* Paste to stdout */

void paste(char *progname, Display *display)
{
    char *buffer;
    int nbytes, num_written = 0;
    
    if ((buffer = XFetchBytes(display, &nbytes)) != NULL) {

	/* Send data to stdout */

	do {
	    num_written += write(STDOUT_FILENO, buffer, nbytes - num_written);
	} while (num_written < nbytes);

	/* Clean up */
	
	XFree(buffer);
    }
}

/*
 * Main program 
 */

int main (int argc, char *argv[])
{
    Display *display;
    char *displayname = NULL, *progname;
    int i, do_paste = 0;

    /* Parse command line arguments */

    progname = argv[0];

    for (i = 1; i < argc; i++) {
        char *arg = argv[i];
        
        if (strcmp("-display", arg) == 0) {
            if (++i >= argc) usage(progname);
            displayname = argv[i];
	    continue;
        } else if (strcmp("-p", arg) == 0) {
	    do_paste = 1;
	    continue;
	}

        fprintf(stderr, "unknown option '%s'\n", arg);
	usage(progname);
    }


    /* Open display */

    if ((display = XOpenDisplay(displayname)) == NULL) {
	fprintf(stderr, "%s: unable to open display %s\n", progname,
		displayname);
	exit(2);
    }

    /* Perform operation */

    if (do_paste) {
	paste(progname, display);
    } else {
	cut(progname, display);
    }

    /* Clean up and exit */

    XCloseDisplay(display);
    exit(0);
}
